# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
# Date: 03.09.2016                                                            #
# Author: Ole-Johan Skrede                                                    #
#                                                                             #
# Solution proposal as part of the exercise program in                        #
# INF4300 - Digital image analysis at the University of Oslo                  #
#                                                                             #
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
"""
Solution proposal for exercise 1, task 3.
"""

# pylint: disable=expression-not-assigned
# pylint: disable=bad-indentation

import os

import numpy as np
import cv2
import matplotlib.pyplot as plt

def plot_image(image, fig_num, name=None, colormap='gray', write_file=None):
  """Plot image"""
  fig = plt.figure(fig_num)
  plt.imshow(image, cmap=colormap, interpolation='none')
  if name is not None:
    plt.title(name)
  plt.xticks([]), plt.yticks([])
  plt.tight_layout()
  if write_file:
    fig.savefig(write_file, bbox_inches='tight', pad_inches=0)
  fig_num += 1
  return fig_num

def main():
  """Main"""
  print('='*80)
  print('Solution to weekly exercises in INF4300')
  print('Exercise 1')
  print('Task 3')
  print('-'*80)

  fig_num = 0
  img_filename = '../../images/coins.png' # Change this to suit you
  write_dir = 'results/images'
  image = cv2.imread(img_filename, cv2.IMREAD_GRAYSCALE)

  # Gloabl thresholding, manually set threshold. These two should achieve the
  # same
  man_t = 128
  thres1 = (image < man_t)*0 + (image >= man_t)*255
  #_, thres1 = cv2.threshold(image, man_t, 255, cv2.THRESH_BINARY)

  # Otsu thresholding
  otsu_t, thres2 = cv2.threshold(image, 0, 255, cv2.THRESH_BINARY+cv2.THRESH_OTSU)

  print('Otsu threshold value: ', otsu_t)

  # Plot the results
  fig_num = plot_image(image, fig_num,
                       write_file=os.path.join(write_dir, 'coins.png'))
  fig = plt.figure(fig_num)
  plt.hist(image.ravel(), 256), plt.title('Gray level histogram')
  fig.savefig(os.path.join(write_dir, 'coins_hist.png'), bbox_inches='tight',
              pad_inches=0)
  fig_num += 1

  fig_num = plot_image(thres1, fig_num,
                       write_file=os.path.join(write_dir, 'coins_thresh_1.png'))
  fig_num = plot_image(thres2, fig_num,
                       write_file=os.path.join(write_dir, 'coins_thresh_2.png'))
  fig_num = plot_image(np.abs(thres1 - thres2), fig_num,
                       write_file=os.path.join(write_dir, 'coins_thresh_diff.png'))


  plt.show()

if __name__ == "__main__":
  main()
